/*
Copyright (c) 2010, Геннадий Берман (http://genberm.narod.ru)
All rights reserved.

Данное программное обеспечение предоставляется на условиях "AS IS".
Разрешается свободное использование, распространение и модификация программы,
с целью ее дальнейшего использования, при условии соблюдения вышеупомянутого
авторского права.

Никакие гарантии не обеспечиваются. Никакой ответственности за прямые,
косвенные или любые другие убытки, возникшие из-за использования данного
программного обеспечения, автор не несет.
*/
#include "mainwindow.h"
#include "qgebcomplextreeview.h"
#include <QRect>
#include <QStandardItemModel>
#include <QVBoxLayout>
#include <QHBoxLayout>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    if (objectName().isEmpty())
        setObjectName(QString::fromUtf8("MainWindow"));
    QWidget * centralWidget = new QWidget(this);
    centralWidget->setObjectName(QString::fromUtf8("centralWidget"));
    setCentralWidget(centralWidget);

    QGridLayout * gridLayout = new QGridLayout;
    centralWidget->setLayout(gridLayout);

    //создаем модель заголовка
    QStandardItemModel *headerModel = new QStandardItemModel;
    BuildHeaderModel(*headerModel);

    complexTreeView = new QgebComplexTreeView(this);
    //gridLayout->addWidget(complexTreeView, 0, 0);
    complexTreeView->setVisible(true);
    complexTreeView->setSortingEnabled(true);
    //передаем в класс модель заголовка (должна быть передана до установки непрокручиваемых колонок)
    complexTreeView->setHeaderModel(headerModel);

    //устанавливаем модель данных
    //последовательность из 5 операторов должна быть фиксированной
    stm = new QStandardItemModel;
    complexTreeView->setModel(stm);
    //таблица 9 колонок
    cols = 9;
    stm->insertColumns(0,cols, QModelIndex());
    complexTreeView->setColumnWidth(0,200);
    //вектор для хранения значений горизонтального выравнивания в колонках
    QVector <int> alignColumnText;
    //вектор для хранения значений вертикального выравнивания в колонках
    QVector <int> valignColumnText;
    //заполняем вектора со сзнчениями выравнивания
    for (int col=0; col<cols; ++col)
    {
        //здесь для примера устанавливаем явно, если это не сделать, то умолчанию (left и top)
        //устанавливаем горизонтальное и вертикальное выравнивание при представлении в HTML по центру
        //(делаем один раз)
        alignColumnText.append(Qt::AlignHCenter);
        valignColumnText.append(Qt::AlignVCenter);
    }
    //сформировать иерархию в колонке 0 тела таблицы на основе QStandardItemModel
    BuildBodyModel();

    //делаем левые две колонки не прокручиваемыми
    complexTreeView->setFrozenColumn(2);

    //разрешаем в случае необходимости горизонтальный и вертикальный ScrollBar
    complexTreeView->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    complexTreeView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    complexTreeView->setObjectName("complexTreeView");  //объектное имя таблицы
    //восстанавливаем сохраненные значения
    gridLayout->addWidget(complexTreeView, 0, 0);
    //!!! пример синхронизации прокручиваемой и не прокручивамой иерархической таблицы
    //раскрываем полную иерархию (сначала у прокручиваемой, а затем не прокручивамой иерархической таблицы)
    complexTreeView->expandAll();
    complexTreeView->getFrozenTable()->expandAll();
}

/*
  Две функции заполнения иерархической таблицы, вторая рекурсивная
*/
void MainWindow::BuildBodyModel()
{
    //заполняем дерево
    //уровень 0, три элемента
    depth = 3; //глубина дерева примера
    currentDepth = 0;
    QString sLevel;
    QString sCol;
    QStandardItem * item;
    QStandardItem * item1;
    for (int row=0; row<3; ++row)
    {
        sLevel = tr("Уровень_") + QString::number(row);
        item = new QStandardItem;
        item->setData(sLevel, Qt::DisplayRole);
        item->insertColumns(0,cols);
        stm->setItem(row, item);
        for (int col=1; col<item->columnCount(); ++col)
        {
            sCol = tr("Кол_") + QString::number(col) + " " + sLevel;
            item1 = new QStandardItem;
            item1->setData(sCol, Qt::DisplayRole);
            stm->setItem(row, col, item1);
        }
        currentDepth ++;
        setNextLevel(item, sLevel);
        currentDepth --;
    }
}


void MainWindow::setNextLevel(QStandardItem * parent, QString pLevel)
{
    if ( currentDepth == depth )
        return;
    QString sLevel;
    QStandardItem * item;
    QString sCol;
    QStandardItem * item1;
    for (int row=0; row<3; ++row)
    {
        sLevel = pLevel + "_" + QString::number(row);
        item = new QStandardItem;
        item->setData(sLevel, Qt::DisplayRole);
        item->insertColumns(0,cols);
        parent->setChild(row, item);
        for (int col=1; col<item->columnCount(); ++col)
        {
            sCol = tr("Кол_") + QString::number(col) + " " + sLevel;
            item1 = new QStandardItem;
            item1->setData(sCol, Qt::DisplayRole);
            parent->setChild(row, col, item1);
        }
        currentDepth ++;
        setNextLevel(item, sLevel);
        currentDepth --;
    }
}

MainWindow::~MainWindow()
{
    int ret = complexTreeView->saveStateTable("c:/complexTreeView.xml");
    if ( ret != 0 )
        QMessageBox::critical(this, tr("Error"), tr("Ошибка сохранения параметров таблицы. Код: ") + QString::number(ret), QMessageBox::Ok);
}

void MainWindow::BuildHeaderModel(QStandardItemModel& headerModel)
{
    //Заголовки "замороженных" колонок. В них не должно быть иерархии, т.е. они всегда на уровне 1.
    QStandardItem* fixedItem = new QStandardItem("Фиксированная\nколонка 1");
    QStandardItem* fixedItem1 = new QStandardItem("Фиксированная\nколонка 2");
    //Два заголовка уровня 1. Для первого будем формировать иерархию, для второго нет.
    QStandardItem* rootItem = new QStandardItem("Уровень 1 с иерархией");
    QStandardItem* rootItem1 = new QStandardItem("Уровень 1\nбез иерархии");

    //список для элементов заголовков, находящихся под rootItem (уровень 2)
    QList<QStandardItem*> list;
        //Создаем элемент уровеня 2 под rootItem. Дочерних не имеет. Название устанавливается с поворотом.
        QStandardItem* rotatedTextCell=new QStandardItem("Уровень 2\nВращаемый текст");
        rotatedTextCell->setData(1, Qt::UserRole);  //Признак поворота: первый параметр равен 1
        list.push_back(rotatedTextCell);
        rootItem->appendColumn(list);
        //Создаем элемент уровня 2 под rootItem. Дочерних не имеет.
        list.clear();
        list.push_back(new QStandardItem("Уровень 2\nОбычный текст"));
        rootItem->appendColumn(list);
        //Создаем элемент уровня 2 под rootItem. Имеет дочерние.
        list.clear();
        QStandardItem* cell=new QStandardItem("Уровень 2\nС дочерними");
        list.push_back(cell);
        rootItem->appendColumn(list);
        //Создаем элемент уровеня 3 под cell.  Дочерних не имеет.
        list.clear();
        list.push_back(new QStandardItem("Уровень 3\nБез дочерних"));
        cell->appendColumn(list);
        //Создаем элемент уровеня 3 под cell.  Имеет дочерние.
        list.clear();
        QStandardItem* cell3=new QStandardItem("Уровень 3\nС дочерними");
        list.push_back(cell3);
        cell->appendColumn(list);
        //Создаем элемент уровеня 4 под cell3.  Дочерних не имеет.
        list.clear();
        list.push_back(new QStandardItem("Уровень 4\nПервый элемент"));
        cell3->appendColumn(list);
        //Создаем элемент уровеня 4 под cell3.  Дочерних не имеет.
        list.clear();
        list.push_back(new QStandardItem("Уровень 4\nВторой элемент"));
        cell3->appendColumn(list);
        //Создаем элемент уровеня 2 под rootItem. Дочерних не имеет.
        list.clear();
        list.push_back(new QStandardItem("Уровень 2\nБез дочерних"));
        rootItem->appendColumn(list);
        //Устанавливаем элементы уровня 1 в модель заголовка
        headerModel.setItem(0, 0, fixedItem);
        headerModel.setItem(0, 1, fixedItem1);
        headerModel.setItem(0, 2, rootItem);
        headerModel.setItem(0, 3, rootItem1);
}
