/*
Copyright (c) 2010, Геннадий Берман (http://genberm.narod.ru)
All rights reserved.

-------------------------------------------------------------------------
Сcылки.
При разработке данного класса заимствованы:
- фрагмент кода из программы построения таблицы со сложным заголовком,
написанной Петром Краснощековым (Krasnoshchekov Petr);
- идея создания не прокручиваемой колонки из программы frozencolumn,
являющейся одним из примеров, поставляемых со свободно распространяемой
библиотекой Qt фирмы Nokia.
-------------------------------------------------------------------------

Данное программное обеспечение предоставляется на условиях "AS IS".
Разрешается свободное использование, распространение и модификация программы,
с целью ее дальнейшего использования, при условии соблюдения вышеупомянутого
авторского права.

Никакие гарантии не обеспечиваются. Никакой ответственности за прямые,
косвенные или любые другие убытки, возникшие из-за использования данного
программного обеспечения, автор не несет.
*/

#ifndef QGEBCOMPLEXTREEVIEW_H
#define QGEBCOMPLEXTREEVIEW_H

#include <QHeaderView>

#include <QProxyModel>
#include <QPointer>

#include <QDialog>
#include <QtGui>
#include <QList>

#include <QKeyEvent>
#include <QStandardItemModel>
#include <QTextStream>
#include <QDomDocument>
#include <QDomAttr>
#include <QStack>


#include <QItemDelegate>
class ItemDelegate : public QItemDelegate
{
public:
    ItemDelegate();
    void paint(QPainter* p, const QStyleOptionViewItem &opt, const QModelIndex &idx) const;

private:
};

class ProxyModelWithHeaderModels: public QProxyModel
{
                Q_OBJECT

                QPointer<QAbstractItemModel> _horizontalHeaderModel;
        public:
                ProxyModelWithHeaderModels(QObject* parent=0);

                QVariant data(const QModelIndex& index, int role=Qt::DisplayRole) const;

                void setHorizontalHeaderModel(QAbstractItemModel* model);

                QAbstractItemModel* horizontalHeaderModel();

};


class HierarchicalHeaderView : public QHeaderView
{
                Q_OBJECT

                class private_data;
                private_data* _pd;

                QStyleOptionHeader styleOptionForCell(int logicalIndex) const;

        protected:
                void paintSection(QPainter *painter, const QRect &rect, int logicalIndex) const;
                QSize sectionSizeFromContents(int logicalIndex) const;
        public:

                enum HeaderDataModelRoles
                        {HorizontalHeaderDataRole=Qt::UserRole,
                         VerticalHeaderDataRole=Qt::UserRole+1};

                HierarchicalHeaderView(Qt::Orientation orientation, QWidget *parent = 0);
                ~HierarchicalHeaderView();

                void setModel(QAbstractItemModel * model);
        private slots:
                void on_sectionResized(int logicalIndex);
};


struct ColumnInfoCTV
{
    QString columnName;
    int columnNumber;
    int columnWidth;
    bool hidden;
    bool frozen;
};


class HideShowColumnsCTV : public QDialog
{
Q_OBJECT
public:
    explicit HideShowColumnsCTV(QList <ColumnInfoCTV *> *columnList, QWidget *parent = 0);


signals:

public slots:

private:
    QVBoxLayout * getVMainLayout();
    QHBoxLayout * getHBottomLayout();
    QGridLayout * getGridLayoutWithTable();

    QList <ColumnInfoCTV *> *columnList;
    QGridLayout * gridLayout;
    QTableWidget * tableWidget;
    QHBoxLayout * hBottomLayout;
    QVBoxLayout * vMainLayout;
    QPushButton * OKPushButton;
    QPushButton * cancelPushButton;

private slots:
    void fixedColumns();
};


class QgebComplexTreeView : public QTreeView
{
    Q_OBJECT

public:
    QgebComplexTreeView(QWidget *parent = 0);
    ~QgebComplexTreeView();

    void setModel(QAbstractItemModel *model);
    QAbstractItemModel * model(){return aiModel;}
    void setFrozenColumn(int frozenCount);
    void setHeaderModel(QStandardItemModel *headerModel);
    QString getPathLocation(){return pathLocation;}
    QString getNameParent(){return this->parent->objectName();}
    int saveStateTable(QString path);
    int restoreStateTable(QString path);
    QString getErrorMessage(){return eMes;}
    QTreeView * getFrozenTable(){return frTblView;}
    void setTitle(QString title){this->title = title;}
    void setPrintHtml(QString textHtml){this->textHtml = textHtml;}
    void setAlignColumn(QVector <int> align);
    void setVAlignColumn(QVector <int> valign);
    void setLevelIndicator(bool levelIndicator=true);
signals:
protected:
      virtual void resizeEvent(QResizeEvent *event);
      virtual QModelIndex moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers);
private:
    void init(QWidget *parent);
    void setHorizontalHeader(HierarchicalHeaderView* hierarchicalHeaderView);
    void setProxyModelWithHeaderModels(ProxyModelWithHeaderModels *proxyModelFixed){this->proxyModelFixed = proxyModelFixed;}
    void updateFrozenTableGeometry();
    void buildList(QStandardItem *standardItem);
    void buildList1(QStandardItem *standardItem);
    void buildColumnList();
    int getBathosModel();
    int buildBathos(QStandardItem *standardItem, int beforeBathos, int maxBathos);
    int getColumnSpan(QStandardItem *standardItem);
    void createHtmlDocument();
    int saveTable(QString pth);
    int save(QString pth);
    QList <ColumnInfoCTV *> * getColumnList(){return columnList;}
    QModelIndex getAppropriateIndex(QModelIndex index);
    QString setHtmlAlign(int col);
    QString setHtmlAlign(QStandardItem * standardItem);
    QString setHtmlFont();
    QString setHtmlFont(QHeaderView * horizontalHeader);
    QString setHtmlFont(QStandardItem * standardItem);
    void buildChildBody(QModelIndex indexParent, QString tab);

    ItemDelegate *delegate;
    QVector<int> alignColumnText;
    QVector<int> valignColumnText;
    bool levelIndicator;
    bool flagSvColumns;
    QString pathLocation;
    int frozenCount;
    int hierColumn;
    QList <ColumnInfoCTV *> *columnList;
    QFrame *frame;
    QAbstractItemModel *aiModel;
    QTreeView *frTblView;
    QWidget *parent;
    HierarchicalHeaderView* hierarchicalHeaderView;
    ProxyModelWithHeaderModels *proxyModelFixed;

    QStandardItemModel *headerModel;
    ProxyModelWithHeaderModels *proxyModel;
    HierarchicalHeaderView* hv;
    QAction *hideColumnAction;
    QAction *outHtmlAction;
    QAction *printTableAction;
    int bathos;
    QTextStream out;
    QDomDocument * doc;
    QString eStr;
    int eLine;
    int eCol;
    QString eMes;
    QString title;
    QString textHtml;
    QStack <QModelIndex>stack;

private slots:
    void setGeometryFixed(int,int,int);
    void updateSectionWidth(int logicalIndex, int, int newSize);
    void updateSectionWidthFromFix(int logicalIndex, int, int newSize);
    void hideColumn();
    void outHtml();
    void printTable();
};

#endif // QGEBCOMPLEXTREEVIEW_H
